@@ -35,7 +35,6 @@
 #' @param x an object of class \code{UThHe} or \code{fissiontracks},
 #'     OR a 2-column matrix with (strictly positive) values and
 #'     uncertainties
-#' @param alpha cutoff value for confidence intervals
 #' @param ... optional arguments
 #' @return
 #' If \code{x} has class \code{UThHe}, returns a list containing the
@@ -61,19 +60,15 @@
 #' \item{p.value}{the p-value of a Chi-square test with \code{df}
 #' degrees of freedom (only reported if \code{model=1}.)}
 #'
-#' \item{age}{a three- or four-element vector with:\cr
+#' \item{age}{a two- or three-element vector with:\cr
 #' \code{t}: the central age.\cr
 #' \code{s[t]}: the standard error of \code{t}.\cr
-#' \code{ci[t]}: the width of a \eqn{100(1-\alpha)\%} confidence
-#' interval for \code{t}.\cr
-#' \code{disp[t]}: the studentised \eqn{100(1-\alpha)\%} confidence
-#' interval enhanced by a factor of \eqn{\sqrt{mswd}} (only reported
-#' if \code{model=1}). }
+#' \code{disp[t]}: the standard error of \code{t} enhanced by a
+#' factor of \eqn{\sqrt{mswd}} (only reported if \code{model=1}). }
 #'
 #' \item{w}{the geological overdispersion term. If \code{model=3},
-#' this is a three-element vector with the standard deviation of the
-#' (assumedly) Normal dispersion and the lower and upper half-widths
-#' of its \eqn{100(1-\alpha)\%} confidence interval. \code{w=0} if
+#' this is a two-element vector with the standard deviation of the
+#' (assumedly) Normal dispersion and its standard error. \code{w=0} if
 #' \code{model<3}.}
 #'
 #' }
@@ -82,18 +77,11 @@
 #'
 #' \describe{
 #'
-#' \item{age}{a three-element vector with:
+#' \item{age}{a two-element vector with the central age and its
+#' standard error.}
 #'
-#' \code{t}: the central age.
-#'
-#' \code{s[t]}: the standard error of \code{t}.
-#'
-#' \code{ci[t]}: the width of a \eqn{100(1-\alpha)\%} confidence
-#' interval for \code{t}.}
-#'
-#' \item{disp}{a three-element vector with the overdispersion
-#' (standard deviation) of the excess scatter, and the upper and lower
-#' half-widths of its \eqn{100(1-\alpha)\%} confidence interval.}
+#' \item{disp}{a two-element vector with the overdispersion (standard
+#' deviation) of the excess scatter, and its standard error.}
 #'
 #' \item{mswd}{the reduced Chi-square statistic of data concordance,
 #' i.e. \eqn{mswd=X^2/df}, where \eqn{X^2} is a Chi-square statistic
@@ -112,9 +100,11 @@
 #' attach(examples)
 #' print(central(UThHe)$age)
 #'
-#' @references Galbraith, R.F. and Laslett, G.M., 1993. Statistical
-#'     models for mixed fission track ages. Nuclear Tracks and
-#'     Radiation Measurements, 21(4), pp.459-470.
+#' @references
+#'
+#' Galbraith, R.F. and Laslett, G.M., 1993. Statistical models for
+#' mixed fission track ages. Nuclear Tracks and Radiation
+#' Measurements, 21(4), pp.459-470.
 #'
 #' Vermeesch, P., 2008. Three new ways to calculate average (U-Th)/He
 #'     ages. Chemical Geology, 249(3), pp.339-347.
@@ -124,7 +114,7 @@
 central <- function(x,...){ UseMethod("central",x) }
 #' @rdname central
 #' @export
-central.default <- function(x,alpha=0.05,...){
+central.default <- function(x,...){
     good <- !is.na(rowSums(x))
     zu <- log(x[good,1])
     su <- x[good,2]/x[good,1]
@@ -139,11 +129,10 @@ central.default <- function(x,alpha=0.05,...){
     # add back one d.o.f. for the homogeneity test
     out$mswd <- Chi2/(out$df+1)
     out$p.value <- 1-stats::pchisq(Chi2,out$df+1)
-    out$age <- c(tt,st,ntfact(alpha)*st)
-    out$disp <- c(fit$sigma,
-                  profile_LL_weightedmean_disp(fit,zu,su,alpha))
-    names(out$age) <- c('t','s[t]','ci[t]')
-    names(out$disp) <- c('s','ll','ul')
+    out$age <- c(tt,st)
+    out$disp <- fit$sigma
+    names(out$age) <- c('t','s[t]')
+    names(out$disp) <- c('w','s[w]')
     out
 }
 #' @param model if the scatter between the data points is solely
@@ -164,25 +153,31 @@ central.default <- function(x,alpha=0.05,...){
 #'
 #' @rdname central
 #' @export
-central.UThHe <- function(x,alpha=0.05,model=1,...){
+central.UThHe <- function(x,model=1,...){
     ns <- nrow(x)
     doSm <- doSm(x)
     fit <- UThHe_logratio_mean(x,model=model,w=0)
     mswd <- mswd_UThHe(x,fit,doSm=doSm)
+    mswd$model <- model # to fulfil requirements of inflate function
+    if (inflate(mswd)){
+        fit$age['disp[t]'] <- uvw2age(fit,doSm=doSm,fact=mswd$mswd)[2]
+    }
     if (model==1){
         out <- c(fit,mswd)
-        out$age['disp[t]'] <-
-            uvw2age(out,doSm=doSm(x),fact=ntfact(alpha,mswd))[2]
     } else if (model==2){
         out <- fit
     } else {
-        w <- get.UThHe.w(x,fit)
+        init <- log(sqrt(mswd$mswd)*fit$age['s[t]']/fit$age['t'])
+        lw <- stats::optimise(LL.uvw,interval=init+c(-5,5),
+                              UVW=fit$uvw,x=x,doSm=doSm(x),
+                              maximum=TRUE)$maximum
+        w <- exp(lw)
+        H <- stats::optimHess(lw,LL.uvw,UVW=fit$uvw,x=x,doSm=doSm(x))
+        sw <- w*solve(-H)
         out <- UThHe_logratio_mean(x,model=model,w=w)
-        out$w <- c(w,profile_LL_central_disp_UThHe(fit=out,x=x,alpha=alpha))
-        names(out$w) <- c('s','ll','ul')
+        out$disp <- c(w,sw)
+        names(out$disp) <- c('w','s[w]')
     }
-    out$alpha <- alpha
-    out$age['ci[t]'] <- ntfact(alpha)*out$age['s[t]']
     out
 }
 
@@ -190,7 +185,7 @@ central.UThHe <- function(x,alpha=0.05,model=1,...){
 #'     the error propagation for the central age?
 #' @rdname central
 #' @export
-central.fissiontracks <- function(x,alpha=0.05,exterr=FALSE,...){
+central.fissiontracks <- function(x,exterr=FALSE,...){
     out <- list()
     if (x$format<2){
         L8 <- lambda('U238')[1]
@@ -219,18 +214,18 @@ central.fissiontracks <- function(x,alpha=0.05,exterr=FALSE,...){
         # add back one d.o.f. for homogeneity test
         out$mswd <- Chi2/(out$df+1)
         out$p.value <- 1-stats::pchisq(Chi2,out$df+1)
-        out$age <- c(tt,st,stats::qt(1-alpha/2,out$df)*st)
-        out$disp <- c(sigma,profile_LL_central_disp_FT(
-                                mu=mu,sigma=sigma,y=Nsj,m=mj,alpha=alpha))
-        names(out$age) <- c('t','s[t]','ci[t]')
-        names(out$disp) <- c('s','ll','ul')
+        out$age <- c(tt,st)
+        kappa <- log(sigma)
+        H <- stats::optimHess(par=c(mu,kappa),fn=LL.FT,y=Nsj,m=mj)
+        out$disp <- c(sigma,sqrt(solve(-H)[2,2])*sigma)
+        names(out$age) <- c('t','s[t]')
+        names(out$disp) <- c('w','s[w]')
     } else if (x$format>1){
         tst <- age(x,exterr=FALSE)
-        out <- central.default(tst,alpha=alpha)
+        out <- central.default(tst)
     }
     if (exterr){
         out$age[1:2] <- add.exterr(x,tt=out$age[1],st=out$age[2])
-        out$age[3] <- out$age[2]*ntfact(alpha)
     }
     out
 }
@@ -238,10 +233,9 @@ central.fissiontracks <- function(x,alpha=0.05,exterr=FALSE,...){
 UThHe_logratio_mean <- function(x,model=1,w=0,fact=1){
     out <- average_uvw(x,model=model,w=w)
     out$model <- model
-    out$w <- w
-    out$age <- rep(NA,3)
-    names(out$age) <- c('t','s[t]','ci[t]')
-    out$age[c('t','s[t]')] <- uvw2age(out,doSm(x),fact=fact)
+    out$disp <- w
+    out$age <- uvw2age(out,doSm(x),fact=fact)
+    names(out$age) <- c('t','s[t]')
     out
 }
 
@@ -288,13 +282,8 @@ average_uvw <- function(x,model=1,w=0){
     out
 }
 
-get.UThHe.w <- function(x,fit){
-    stats::optimize(LL.uvw,interval=c(0,100),
-                    UVW=fit$uvw,x=x,doSm=doSm(x),
-                    maximum=TRUE)$maximum
-}
-
-LL.uvw <- function(w,UVW,x,doSm=TRUE,LL=TRUE){
+LL.uvw <- function(lw,UVW,x,doSm=TRUE,LL=TRUE){
+    w <- exp(lw)
     out <- 0
     for (i in 1:length(x)){
         if (doSm){
@@ -318,7 +307,7 @@ mswd_UThHe <- function(x,fit,doSm=FALSE){
     out <- list()
     if (doSm) nd <- 3
     else nd <- 2
-    SS <- LL.uvw(w=fit$w,fit$uvw[1:nd],x,doSm=doSm,LL=FALSE)
+    SS <- LL.uvw(lw=log(fit$disp),fit$uvw[1:nd],x,doSm=doSm,LL=FALSE)
     out$df <- nd*(length(x)-1)
     out$mswd <- as.numeric(SS/out$df)
     out$p.value <- 1-stats::pchisq(SS,out$df)
@@ -331,9 +320,10 @@ continuous_mixture <- function(zu,su){
         for (i in 1:30){ # page 100 of Galbraith (2005)
             wu <- 1/(sigma^2+su^2)
             mu <- sum(wu*zu,na.rm=TRUE)/sum(wu,na.rm=TRUE)
-            fit <- stats::optimise(f=eq.6.9,interval=c(0,10*stats::sd(zu)),
+            kappa <- log(sigma)
+            fit <- stats::optimise(f=eq.6.9,interval=kappa+c(-3,3),
                                    mu=mu,zu=zu,su=su)
-            sigma <- fit$minimum
+            sigma <- exp(fit$minimum)
         }
     } else{
         sigma <- 0
@@ -342,12 +332,14 @@ continuous_mixture <- function(zu,su){
     }
     out <- list()
     smu <- 1/sqrt(sum(wu,na.rm=TRUE))
+    ssigma <- 1/sqrt(2*(sigma^2)*sum(wu^2,na.rm=TRUE))
     out$mu <- c(mu,smu)
-    out$sigma <- sigma
+    out$sigma <- c(sigma,ssigma)
     out
 }
 
-eq.6.9 <- function(sigma,mu,zu,su){
+eq.6.9 <- function(kappa,mu,zu,su){
+    sigma <- exp(kappa)
     wu <- 1/(sigma^2+su^2)
     (1-sum((wu*(zu-mu))^2,na.rm=TRUE)/sum(wu,na.rm=TRUE))^2
 }
