@@ -114,7 +114,7 @@
 peakfit <- function(x,...){ UseMethod("peakfit",x) }
 #' @rdname peakfit
 #' @export
-peakfit.default <- function(x,k='auto',sigdig=2,oerr=3,log=TRUE,np=4,...){
+peakfit.default <- function(x,k='auto',sigdig=2,oerr=3,log=TRUE,np=3,...){
     good <- !is.na(x[,1]+x[,2])
     X <- subset(x,subset=good)
     if (k<1) return(NULL)
@@ -139,7 +139,7 @@ peakfit.default <- function(x,k='auto',sigdig=2,oerr=3,log=TRUE,np=4,...){
 #' @rdname peakfit
 #' @export
 peakfit.fissiontracks <- function(x,k=1,exterr=TRUE,sigdig=2,
-                                  log=TRUE,oerr=3,np=4,...){
+                                  log=TRUE,oerr=3,np=3,...){
     out <- NULL
     if (k == 0) return(out)
     if (identical(k,'auto')) k <- BIC_fit(x,5,log=log)
@@ -196,7 +196,7 @@ peakfit.fissiontracks <- function(x,k=1,exterr=TRUE,sigdig=2,
 #' @export
 peakfit.UPb <- function(x,k=1,type=4,cutoff.76=1100,
                         cutoff.disc=discfilter(),common.Pb=0,
-                        exterr=TRUE,sigdig=2,log=TRUE,oerr=3,np=4,...){
+                        exterr=TRUE,sigdig=2,log=TRUE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,type=type,cutoff.76=cutoff.76,
                    cutoff.disc=cutoff.disc,exterr=exterr,
                    sigdig=sigdig,log=log,oerr=oerr,
@@ -205,7 +205,7 @@ peakfit.UPb <- function(x,k=1,type=4,cutoff.76=1100,
 #' @rdname peakfit
 #' @export
 peakfit.PbPb <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,common.Pb=0,oerr=3,np=4,...){
+                         log=TRUE,common.Pb=0,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,log=log,
                    common.Pb=common.Pb,oerr=oerr,np=np,...)
 }
@@ -221,49 +221,49 @@ peakfit.PbPb <- function(x,k=1,exterr=TRUE,sigdig=2,
 #' @rdname peakfit
 #' @export
 peakfit.ArAr <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,i2i=FALSE,oerr=3,np=4,...){
+                         log=TRUE,i2i=FALSE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
 #' @rdname peakfit
 #' @export
 peakfit.ThPb <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,i2i=FALSE,oerr=3,np=np,...){
+                         log=TRUE,i2i=FALSE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
-                   log=log,i2i=i2i,oerr=oerr,np=4,...)
+                   log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
 #' @rdname peakfit
 #' @export
 peakfit.KCa <- function(x,k=1,exterr=TRUE,sigdig=2,
-                        log=TRUE,i2i=FALSE,oerr=3,np=4,...){
+                        log=TRUE,i2i=FALSE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
 #' @rdname peakfit
 #' @export
 peakfit.ReOs <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,i2i=TRUE,oerr=3,np=4,...){
+                         log=TRUE,i2i=TRUE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
 #' @rdname peakfit
 #' @export
 peakfit.SmNd <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,i2i=TRUE,oerr=3,np=4,...){
+                         log=TRUE,i2i=TRUE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
 #' @rdname peakfit
 #' @export
 peakfit.RbSr <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,i2i=TRUE,oerr=3,np=4,...){
+                         log=TRUE,i2i=TRUE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
 #' @rdname peakfit
 #' @export
 peakfit.LuHf <- function(x,k=1,exterr=TRUE,sigdig=2,
-                         log=TRUE,i2i=TRUE,oerr=3,np=4,...){
+                         log=TRUE,i2i=TRUE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,i2i=i2i,oerr=oerr,np=np,...)
 }
@@ -288,18 +288,18 @@ peakfit.LuHf <- function(x,k=1,exterr=TRUE,sigdig=2,
 #' @rdname peakfit
 #' @export
 peakfit.ThU <- function(x,k=1,exterr=FALSE,sigdig=2,
-                        log=TRUE,oerr=3,Th0i=0,np=4,...){
+                        log=TRUE,oerr=3,Th0i=0,np=3,...){
     peakfit_helper(x,k=k,exterr=exterr,sigdig=sigdig,
                    log=log,oerr=oerr,Th0i=Th0i,np=np,...)
 }
 #' @rdname peakfit
 #' @export
-peakfit.UThHe <- function(x,k=1,sigdig=2,log=TRUE,oerr=3,np=4,...){
+peakfit.UThHe <- function(x,k=1,sigdig=2,log=TRUE,oerr=3,np=3,...){
     peakfit_helper(x,k=k,sigdig=sigdig,log=log,oerr=oerr,np=np,...)
 }
 peakfit_helper <- function(x,k=1,type=4,cutoff.76=1100,cutoff.disc=discfilter(),
                            exterr=TRUE,sigdig=2,log=TRUE,i2i=FALSE,
-                           common.Pb=0,oerr=3,Th0i=0,np=4,...){
+                           common.Pb=0,oerr=3,Th0i=0,np=3,...){
     if (k<1) return(NULL)
     if (identical(k,'auto')){
         k <- BIC_fit(x,5,log=log,type=type,cutoff.76=cutoff.76,i2i=i2i,
@@ -367,7 +367,9 @@ get.props.err <- function(E){
 
 peaks2legend <- function(fit,k=NULL,sigdig=2,oerr=3){
     if (identical(k,'min')){
-        tit <- peaktit(x=fit$peaks[1],sx=fit$peaks[2],p=fit$props[1],
+        if ('mu'%in%names(fit)) p <- fit$props[1] # 4 par model
+        else p <- NULL # 3 par model
+        tit <- peaktit(x=fit$peaks[1],sx=fit$peaks[2],p=p,
                        sigdig=sigdig,oerr=oerr,prefix=paste0('Minimum:'))
         out <- as.expression(tit)
     } else {
@@ -541,7 +543,7 @@ BIC_fit <- function(x,max.k,...){
     }) 
 }
 
-min_age_model <- function(zs,np=4){
+min_age_model <- function(zs,np=3){
     # maps the parameters from -Inf/+Inf to model space
     mappar <- function(par,Mz){
         np <- length(par)
@@ -567,9 +569,10 @@ min_age_model <- function(zs,np=4){
     H <- tryCatch(stats::optimHess(fit$par,LL,zs=zs,Mz=Mz),
                   error=function(e){ return(NULL) })
     if (is.null(H) & np>3){
+        warning('Non-invertible Hessian, reduced number of parameters from 4 to 3')
         return(min_age_model(zs=zs,np=3))
     }
-    lE <- MASS::ginv(H)
+    lE <- inverthess(H)
     par <- mappar(fit$par,Mz)
     # propagate the uncertainties from -Inf/+Inf to model space
     J <- diag(np)
@@ -581,6 +584,7 @@ min_age_model <- function(zs,np=4){
     }
     E <- J %*% lE %*% t(J)
     if (E[1,1]<0 & np==4){
+        warning('Singular covariance matrix, reduced number of parameters from 4 to 3')
         out <- min_age_model(zs=zs,np=3)
     } else {
         out <- list()
